]> git.ipfire.org Git - thirdparty/git.git/blobdiff - bisect.c
t4216: avoid unnecessary subshell in test_bloom_filters_not_used
[thirdparty/git.git] / bisect.c
index f5ce3a4b70d536f4d3cc0ab93c914a7172f03787..9154f810f76af3cdbf82b8bb509f47aa392b0e01 100644 (file)
--- a/bisect.c
+++ b/bisect.c
@@ -761,7 +761,7 @@ static struct commit **get_bad_and_good_commits(struct repository *r,
        return rev;
 }
 
-static void handle_bad_merge_base(void)
+static enum bisect_error handle_bad_merge_base(void)
 {
        if (is_expected_rev(current_bad_oid)) {
                char *bad_hex = oid_to_hex(current_bad_oid);
@@ -782,14 +782,14 @@ static void handle_bad_merge_base(void)
                                "between %s and [%s].\n"),
                                bad_hex, term_bad, term_good, bad_hex, good_hex);
                }
-               exit(3);
+               return BISECT_MERGE_BASE_CHECK;
        }
 
        fprintf(stderr, _("Some %s revs are not ancestors of the %s rev.\n"
                "git bisect cannot work properly in this case.\n"
                "Maybe you mistook %s and %s revs?\n"),
                term_good, term_bad, term_good, term_bad);
-       exit(1);
+       return BISECT_FAILED;
 }
 
 static void handle_skipped_merge_base(const struct object_id *mb)
@@ -830,7 +830,8 @@ static enum bisect_error check_merge_bases(int rev_nr, struct commit **rev, int
        for (; result; result = result->next) {
                const struct object_id *mb = &result->item->object.oid;
                if (oideq(mb, current_bad_oid)) {
-                       handle_bad_merge_base();
+                       res = handle_bad_merge_base();
+                       break;
                } else if (0 <= oid_array_lookup(&good_revs, mb)) {
                        continue;
                } else if (0 <= oid_array_lookup(&skipped_revs, mb)) {
@@ -975,10 +976,10 @@ void read_bisect_terms(const char **read_bad, const char **read_good)
 }
 
 /*
- * We use the convention that exiting with an exit code 10 means that
+ * We use the convention that return BISECT_INTERNAL_SUCCESS_1ST_BAD_FOUND (-10) means
  * the bisection process finished successfully.
- * In this case the calling shell script should exit 0.
- *
+ * In this case the calling function or command should not turn a
+ * BISECT_INTERNAL_SUCCESS_1ST_BAD_FOUND return code into an error or a non zero exit code.
  * If no_checkout is non-zero, the bisection process does not
  * checkout the trial commit but instead simply updates BISECT_HEAD.
  */
@@ -1009,23 +1010,25 @@ enum bisect_error bisect_next_all(struct repository *r, const char *prefix, int
 
        if (!revs.commits) {
                /*
* We should exit here only if the "bad"
                 * commit is also a "skip" commit.
                 */
                res = error_if_skipped_commits(tried, NULL);
                if (res < 0)
-                       exit(-res);
+                       return res;
                printf(_("%s was both %s and %s\n"),
                       oid_to_hex(current_bad_oid),
                       term_good,
                       term_bad);
-               exit(1);
+
+               return BISECT_FAILED;
        }
 
        if (!all) {
                fprintf(stderr, _("No testable commit found.\n"
                        "Maybe you started with bad path parameters?\n"));
-               exit(4);
+
+               return BISECT_NO_TESTABLE_COMMIT;
        }
 
        bisect_rev = &revs.commits->item->object.oid;
@@ -1033,12 +1036,19 @@ enum bisect_error bisect_next_all(struct repository *r, const char *prefix, int
        if (oideq(bisect_rev, current_bad_oid)) {
                res = error_if_skipped_commits(tried, current_bad_oid);
                if (res)
-                       exit(-res);
+                       return res;
                printf("%s is the first %s commit\n", oid_to_hex(bisect_rev),
                        term_bad);
+
                show_diff_tree(r, prefix, revs.commits->item);
-               /* This means the bisection process succeeded. */
-               exit(10);
+               /*
+                * This means the bisection process succeeded.
+                * Using BISECT_INTERNAL_SUCCESS_1ST_BAD_FOUND (-10)
+                * so that the call chain can simply check
+                * for negative return values for early returns up
+                * until the cmd_bisect__helper() caller.
+                */
+               return BISECT_INTERNAL_SUCCESS_1ST_BAD_FOUND;
        }
 
        nr = all - reaches - 1;
@@ -1009,23 +1010,25 @@ enum bisect_error bisect_next_all(struct repository *r, const char *prefix, int
        if (!revs.commits) {
                /*
-                * We should exit here only if the "bad"
+                * We should return error here only if the "bad"
                 * commit is also a "skip" commit.
                 */